package com.example.hi_teacher_screenshot_listener;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Build;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import java.util.HashMap;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** HiTeacherScreenshotListenerPlugin */
public class HiTeacherScreenshotListenerPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private EventChannel.EventSink eventSink;
  private ScreenshotHelper screenshotHelper;

  private Activity activity;

  static final String CHANNEL_NAME = "dev.hiteacher.screenshot_listener/bridge";
  static final String EVENT_CHANNEL_NAME = "dev.hiteacher.screenshot_listener/listener";

  @RequiresApi(api = Build.VERSION_CODES.TIRAMISU)
  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), CHANNEL_NAME);
    channel.setMethodCallHandler(this);
    EventChannel eventChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(), EVENT_CHANNEL_NAME);
    eventChannel.setStreamHandler(new EventChannel.StreamHandler() {
      @Override
      public void onListen(Object o, EventChannel.EventSink tempEventSink) {
        eventSink = tempEventSink;
      }

      @Override
      public void onCancel(Object o) {
        eventSink = null;
        screenshotHelper.stop();
      }
    });
    Context applicationContext = flutterPluginBinding.getApplicationContext();
    screenshotHelper = new ScreenshotHelper(applicationContext);
    screenshotHelper.setScreenshotListener(data -> {
      if (eventSink != null) {
        eventSink.success(data);
      }
    });

  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("start")) {
      String failReason = "";
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
        final int permission = ContextCompat.checkSelfPermission(activity.getApplicationContext(), android.Manifest.permission.READ_MEDIA_IMAGES);
        if (permission != PackageManager.PERMISSION_GRANTED) {
          ActivityCompat.requestPermissions(activity, new String[]{android.Manifest.permission.READ_MEDIA_IMAGES}, 0);
        }
        if (permission == PackageManager.PERMISSION_DENIED) {
          failReason = "访问相册权限被拒";
        }
      }
      screenshotHelper.start();
      HashMap<String, Object> map = new HashMap<>();
      map.put("success", failReason.isEmpty());
      map.put("message", failReason);
      result.success(map);
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding activityPluginBinding) {
    activity = activityPluginBinding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding activityPluginBinding) {
    onAttachedToActivity(activityPluginBinding);
  }

  @Override
  public void onDetachedFromActivity() {

  }
}
