import 'dart:async';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:hi_teacher_screenshot_listener/hi_teacher_screenshot_listener.dart';

class ListenPage extends StatefulWidget {
  const ListenPage({Key? key}) : super(key: key);

  @override
  State<ListenPage> createState() => _ListenPageState();
}

class _ListenPageState extends State<ListenPage> {
  final _screenshotListenerPlugin = HiTeacherScreenshotListener();
  StreamSubscription? _screenshotSubscription;
  String _screenshotPath = '';

  @override
  void initState() {
    super.initState();
    _screenshotSubscription =
        _screenshotListenerPlugin.getUserScreenshot().listen(
              (screenshotPath) {
            if (screenshotPath != null) {
              setState(() {
                _screenshotPath = screenshotPath;
              });
              debugPrint(screenshotPath);
            }
          },
        );
  }

  @override
  void dispose() {
    _screenshotSubscription?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Listen Page'),
          leading: IconButton(
            icon: const Icon(Icons.arrow_back_ios),
            onPressed: () {
              Navigator.pop(context);
            },
          ),
        ),
        body: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            _screenshotPath.isEmpty
                ? const SizedBox()
                : Image.file(
              File(_screenshotPath),
              height: 200,
              width: 200,
            ),
            Row(),
            ElevatedButton(
              onPressed: () async {
                final result = await _screenshotListenerPlugin.startListen();
                debugPrint(
                    '------> success = ${result['success']} message = ${result['message']}');
              },
              child: const Text('开始监听'),
            ),
          ],
        ),
      ),
    );
  }
}
