#import "HiTeacherScreenshotListenerPlugin.h"

static  NSString * const methodChannelName = @"dev.hiteacher.screenshot_listener/bridge";
static NSString * const screenshotChannelName = @"dev.hiteacher.screenshot_listener/listener";

@interface HiTeacherScreenshotListenerPlugin()<FlutterStreamHandler>
@property (nonatomic, copy, nonnull) FlutterEventSink eventSink;
@end

@implementation HiTeacherScreenshotListenerPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:methodChannelName
                                                                binaryMessenger:[registrar messenger]];
    HiTeacherScreenshotListenerPlugin* instance = [[HiTeacherScreenshotListenerPlugin alloc] init];
    
    FlutterEventChannel *eventChannel = [FlutterEventChannel eventChannelWithName:screenshotChannelName
                                                                  binaryMessenger:[registrar messenger]];
    [eventChannel setStreamHandler: instance];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)startListen {
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(userDidTakeScreenshot:) name:UIApplicationUserDidTakeScreenshotNotification
                                               object:nil];
}

- (void)userDidTakeScreenshot:(NSNotification *)notification {
    UIImage *image = [self dataWithScreenshotInPNGFormat];
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSString *tempDir = NSTemporaryDirectory();
        NSError *error = nil;
        NSDate* date = [NSDate dateWithTimeIntervalSinceNow:0];
        NSInteger time = (NSInteger)[date timeIntervalSince1970]*1000;
        NSString *filePath = [NSString stringWithFormat:@"%@/%zd.jpeg", tempDir, time];
        [UIImageJPEGRepresentation(image, 1.0) writeToFile:filePath
                                                   options:NSDataWritingAtomic
                                                     error:&error];
        if (error == nil) {
            self.eventSink(filePath);
        }
    });
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"start" isEqualToString:call.method]) {
        [self startListen];
        result(@{@"success": @YES, @"message" : @""});
    }
    else {
        result(FlutterMethodNotImplemented);
    }
}

- (FlutterError * _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    _eventSink = nil;
    [self cancelListen];
    return nil;
}

- (FlutterError * _Nullable)onListenWithArguments:(id _Nullable)arguments eventSink:(nonnull FlutterEventSink)events {
    if (events) {
        _eventSink = events;
    }
    return nil;
}

- (UIImage *)dataWithScreenshotInPNGFormat
{
    CGSize imageSize = CGSizeZero;
    UIInterfaceOrientation orientation = [UIApplication sharedApplication].statusBarOrientation;
    if (UIInterfaceOrientationIsPortrait(orientation))
        imageSize = [UIScreen mainScreen].bounds.size;
    else
        imageSize = CGSizeMake([UIScreen mainScreen].bounds.size.height, [UIScreen mainScreen].bounds.size.width);
    
    UIGraphicsBeginImageContextWithOptions(imageSize, NO, 0);
    CGContextRef context = UIGraphicsGetCurrentContext();
    for (UIWindow *window in [[UIApplication sharedApplication] windows])
    {
        CGContextSaveGState(context);
        CGContextTranslateCTM(context, window.center.x, window.center.y);
        CGContextConcatCTM(context, window.transform);
        CGContextTranslateCTM(context, -window.bounds.size.width * window.layer.anchorPoint.x, -window.bounds.size.height * window.layer.anchorPoint.y);
        if (orientation == UIInterfaceOrientationLandscapeLeft)
        {
            CGContextRotateCTM(context, M_PI_2);
            CGContextTranslateCTM(context, 0, -imageSize.width);
        }
        else if (orientation == UIInterfaceOrientationLandscapeRight)
        {
            CGContextRotateCTM(context, -M_PI_2);
            CGContextTranslateCTM(context, -imageSize.height, 0);
        } else if (orientation == UIInterfaceOrientationPortraitUpsideDown) {
            CGContextRotateCTM(context, M_PI);
            CGContextTranslateCTM(context, -imageSize.width, -imageSize.height);
        }
        if ([window respondsToSelector:@selector(drawViewHierarchyInRect:afterScreenUpdates:)])
        {
            [window drawViewHierarchyInRect:window.bounds afterScreenUpdates:YES];
        }
        else
        {
            [window.layer renderInContext:context];
        }
        CGContextRestoreGState(context);
    }
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    return image;
}

- (void)cancelListen {
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIApplicationUserDidTakeScreenshotNotification
                                                  object:nil];
}

- (void)dealloc
{
    [self cancelListen];
}


@end
